/**
 * Implements the back end interface of grouse.  This class essentially
 * takes the QT callbacks and links it to the grouse code.
 */
#if (__GNUC__ < 3)
#  include <hash_map>
#else
#  include <ext/hash_map>
#endif
#include <string>
#include <tulip/Graph.h>
#include <tulip/GWInteractor.h>
#include "GrouseInterfaceData.h"
#include "MetaNodeBoundingCircle.h"
#include "MouseGrouseNavigate.h"
#include "OpenCloseHierarchy.h"
#include "GrouseSelectionProperty.h"

class GrouseInterfaceBackEnd : public GrouseInterfaceData {
  Q_OBJECT;
 public:
  GrouseInterfaceBackEnd (QWidget *parent = 0, const char *name = NULL,
			  const char *WFlags = NULL) :
    boundingCircles (NULL), scaleMode (false),
    labelToggle(false) {}
  ~GrouseInterfaceBackEnd () {
    delete mouse;
    delete boundingCircles;
    delete grouseSelectionProperty;
  };

  //================================================================
  /**
   * finds a graph with the passed name.  If the name is not found
   * returns a null pointer.
   */
  Graph *findGraph (const std::string &graphName);
  
  //================================================================
  /**
   * The function initializes the rendering widgets to their initial
   * startup state after a file has been loaded.  The graph just
   * loaded is passed as the parameter.
   */
  void initInterface (Graph *graph);

  //================================================================
  /**
   * Sets up the hierarchy variable
   */
  void initHierarchy ();

  //================================================================
  /**
   * Sets up the finalLayoutVariable
   */
  void initFinalLayout ();

 signals:
  void graphWidgetSelectionChangedSignal (tlp::BooleanProperty &selection);

 private slots:
  void metaGraphHierarchyWidget1_newRightButtonPath (const std::vector<tlp::node> &newPath);
  void metaGraphHierarchyWidget1_selectionChanged(const std::vector<tlp::node> &newHierNodes, const std::vector<tlp::Graph *> &newHierGraphs);
  void graphWidgetSelectionChanged (tlp::BooleanProperty &selection);

 private:
  bool scaleMode;
  bool labelToggle;
  MetaNodeBoundingCircles *boundingCircles;
  MouseGrouseNavigate *mouse;
  Graph *rootGraph;
  tlp::node hierarchyRootNode;
  OpenCloseHierarchy *openCloseHierarchy;
  GrouseSelectionProperty *grouseSelectionProperty;
  Graph *finalLayout;
  Graph *hierarchy;
  Graph *finalLayoutDisplay;
  void fileOpen();
  void fileExit();
  void fileSaveAs();
  void fileScale_Mode();
  void fileLabelToggle();
  void qtConnections();
};
