#include <string>
#include <tulip/DoubleProperty.h>
#include <tulip/ColorProperty.h>
#include "GrouseSelectionProperty.h" 

using namespace tlp;
using namespace std;

const string viewBorderWidth ("viewBorderWidth");
const string viewBorderColor ("viewBorderColor");

//========================================================================
GrouseSelectionProperty::
GrouseSelectionProperty (Graph *displayGraph,
			 Graph *hierarchy, 
			 OpenCloseHierarchy *openCloseHierarchy) :
  displayGraph(displayGraph),
  hierarchy(hierarchy),
  openCloseHierarchy(openCloseHierarchy)
{
  selection = new BooleanProperty (hierarchy);
  selection->setAllNodeValue (false);
  selection->setAllEdgeValue (false);
}//end constructor

//========================================================================
GrouseSelectionProperty::~GrouseSelectionProperty() {
  delete selection;
}

//========================================================================
void GrouseSelectionProperty::clearSelection () {
  //set all node displays to unselected
  DoubleProperty *borderWidth = 
    displayGraph->getProperty<DoubleProperty> (viewBorderWidth);
  borderWidth->setAllNodeValue (META_NODE_BORDER_WIDTH);
  ColorProperty *borderColor =
    displayGraph->getProperty<ColorProperty> (viewBorderColor);
  borderColor->setAllNodeValue (META_NODE_BORDER_COLOUR);
  
  //unselect all augmented displays
  Iterator<node> *selectionIterator = selection->getNodesEqualTo (true);
  while (selectionIterator->hasNext()) {
    GlABoundingCircle *curCircle = 
      openCloseHierarchy->getBoundingCircle (selectionIterator->next());
    if (curCircle != NULL) 
      curCircle->setSelected (false);
  } delete selectionIterator; //end while

  //set all proxies false
  selection->setAllNodeValue (false);
  selection->setAllEdgeValue (false);
}//end clear selection

//========================================================================
void GrouseSelectionProperty::selectNode (const node &curNode) {
  DoubleProperty *borderWidth = 
    displayGraph->getProperty<DoubleProperty> (viewBorderWidth);
  borderWidth->setNodeValue (curNode, SELECTED_BORDER_WIDTH);
  ColorProperty *borderColor =
    displayGraph->getProperty<ColorProperty> (viewBorderColor);
  borderColor->setNodeValue (curNode, SELECTION_COLOUR);
  selection->setNodeValue (openCloseHierarchy->getHierarchyNode (curNode), 
			   true);
}

//========================================================================
void GrouseSelectionProperty::selectAug (GlABoundingCircle *aug) {
  aug->setSelected (true);
  selection->setNodeValue (aug->getHierarchyNode(), true);
}

//========================================================================
void GrouseSelectionProperty::setSelection (const vector<node> &nodes,
					    vector<GlABoundingCircle *> &augs,
					    bool emitSignal) {
  clearSelection();
  for (vector<node>::const_iterator nodeIt = nodes.begin();
       nodeIt != nodes.end(); ++nodeIt)
    selectNode (*nodeIt);
  for (vector<GlABoundingCircle *>::iterator augsIt = augs.begin();
       augsIt != augs.end(); ++augsIt) 
    selectAug (*augsIt);
  if (emitSignal) emit selectionChanged (*selection);
}
