#ifndef MOUSE_GROUSE_INTERFACE_H
#define MOUSE_GROUSE_INTERFACE_H
#include <qevent.h>
#include <tulip/GlGraphWidget.h>
#include <tulip/GWInteractor.h>
#include <tulip/Morphing.h>
#include <tulip/MetaGraphHierarchyWidget.h>
#include <qlistview.h>
#include "OpenCloseHierarchy.h"
#include "MetaNodeBoundingCircle.h"
#include "GrouseSelectionProperty.h"


//================================================================
/**
 * The following class sets up the mouse interface for the
 * Grouse project.  The interface maps all mouse clicks to 
 * actions within grouse along with keybord actions as well.
 */
class MouseGrouseNavigate : public GWInteractor {
 public:
  MouseGrouseNavigate(OpenCloseHierarchy *openCloseHierarchy,
		      MetaGraphHierarchyWidget *metaGraphHierarchy,
		      GrouseSelectionProperty *grouseSelectionProperty);
  ~MouseGrouseNavigate();
  bool eventFilter (QObject *, QEvent *);
  GWInteractor * clone() { 
    return new MouseGrouseNavigate(openCloseHierarchy, metaGraphHierarchy,
				   grouseSelectionProperty);
  }
  void morphOpen (GlGraphWidget *graphWidget,
		  const tlp::node &pickedNode);
  void morphClosed (GlGraphWidget *graphWidget,
		    GlABoundingCircle *pickedBoundingCircle);
  void powerOpenNodes (const node &displayNode, 
		       GlGraphWidget *graphWidget);

 private:
  QPoint startDrag;
  MetaGraphHierarchyWidget *metaGraphHierarchy;
  GrouseSelectionProperty *grouseSelectionProperty;
  inline void initStartDrag (const QPoint &start) { startDrag = start; }
  void pan (const QPoint &end, GlGraphWidget *graph);
  void select (const QPoint &mouseClick, GlGraphWidget *graphWidget);
  void openClose (const QPoint &mouseClick, GlGraphWidget *graphWidget);
  void powerOpen (const QPoint &mouseClick,
		  GlGraphWidget *graphWidget);
  OpenCloseHierarchy *openCloseHierarchy;
  void mPressEvent(GlGraphWidget *,QMouseEvent *);
  void mReleaseEvent(GlGraphWidget *,QMouseEvent *);
  void mMoveEvent(GlGraphWidget *,QMouseEvent *);
  void keyPressEvent (GlGraphWidget *, QKeyEvent *);
  void keyReleaseEvent(GlGraphWidget *, QKeyEvent *);
  void wheelEvent(GlGraphWidget *, QWheelEvent *);
  void morphCamera (GlGraphWidget *graphWidget,
		    const Coord &nodePos, const double nodeRadius);
  void morphGraph (GlGraphWidget *graphWidget, 
		   tlp::GraphState *g0, 
		   tlp::GraphState *g1,
		   stdext::hash_map<tlp::node, unsigned int> &order);
  void nodesAndEdgesToRemove (tlp::Graph *graphToBeClosed,
			      std::vector<node> &nodesRemoved,
			      std::vector<edge> &edgesRemoved);
  
};
#endif
