#include <tulip/IntegerProperty.h>
#include <tulip/LayoutProperty.h>
#include <tulip/SizeProperty.h>
#include <tulip/DoubleProperty.h>
#include <tulip/StringProperty.h>
#include <tulip/ExtendedClusterOperation.h>
#include <tulip/ForEach.h>
#include <tulip/DrawingTools.h>
#include "OpenClose.h"

using namespace std;
using namespace tlp;

//================================================================
node OpenClose::close (Graph *displayGraph, Graph *subgraph,
		       const Coord &pos,
		       const int shape, const Color &color, 
		       const double borderWidth, const Size &size,
		       const string &label) {
  set<node> tmp;
  node curNode;
  forEach (curNode, subgraph->getNodes()) tmp.insert (curNode);
  node groupedMetaNode = tlp::createMetaNode (displayGraph, tmp,
					      closedSubgraphs,
					      groupedMetaGraphs);
  IntegerProperty *shapes = 
    displayGraph->getProperty<IntegerProperty>("viewShape");
  shapes->setNodeValue (groupedMetaNode, shape);
  ColorProperty *colours = 
    displayGraph->getProperty<ColorProperty>("viewColor");
  colours->setNodeValue (groupedMetaNode, color);
  DoubleProperty *borders =
    displayGraph->getProperty<DoubleProperty> ("viewBorderWidth");
  borders->setNodeValue (groupedMetaNode, borderWidth);
  SizeProperty *sizes = displayGraph->getProperty<SizeProperty> ("viewSize");
  sizes->setNodeValue (groupedMetaNode, size);
  StringProperty *labels =
    displayGraph->getProperty<StringProperty> ("viewLabel");
  labels->setNodeValue (groupedMetaNode, label);
  
  return groupedMetaNode;
}//end close

//================================================================
void OpenClose::open (Graph *displayGraph, node metaNode) {
  SizeProperty *viewSize =
    displayGraph->getProperty<SizeProperty> ("viewSize");
  Graph *metaGraph = groupedMetaGraphs->getNodeValue (metaNode);
  pair<Coord, Coord> boundingBox = 
    tlp::computeBoundingBox (metaGraph, 
			     metaGraph
			     ->getProperty<LayoutProperty> ("viewLayout"),
			     metaGraph
			     ->getProperty<SizeProperty> ("viewSize"),
			     metaGraph
			     ->getProperty<DoubleProperty> ("viewRotation"));
  Coord c_size = boundingBox.first - boundingBox.second;
  Size size (c_size.getX(), c_size.getY(), c_size.getZ());
  viewSize->setNodeValue (metaNode, size);
  
  tlp::openMetaNode (displayGraph, metaNode, closedSubgraphs,
		     groupedMetaGraphs);
}//end open
