#if (QT_REL == 4)
#include <QtGui/qapplication.h>
#else
#include <qapplication.h>
#include <qimage.h>
#endif

#include <tulip/Graph.h>
#include <tulip/TlpTools.h>
#include <tulip/GlGraphWidget.h>
#include <tulip/MouseObject.h>
#include <tulip/LayoutProperty.h>
#include <tulip/ColorProperty.h>
#include <tulip/DoubleProperty.h>
#include <tulip/Morphing.h>

using namespace std;
using namespace tlp;

const unsigned int NB_MESURE = 75;
/*******************************************************************/
void importGraph(const string &filename, GlGraph *render) {
  DataSet dataSet;
  dataSet.set("file::filename", filename);
  Graph *newGraph=tlp::importGraph("tlp", dataSet, NULL);
  if (newGraph!=0) {
    render->setGraph(newGraph);
    
    DataSet glGraphData;
    if (dataSet.get<DataSet>("displaying", glGraphData))
      render->setParameters(glGraphData);
  }
  render->setEdge3D(true);
  render->setEdgeSizeInterpolate(false);
}
/*******************************************************************/
void makeFilm(GlGraphWidget *glW, QApplication *app) {
  Graph *graph = glW->getGraph();
  
  LayoutProperty *layout = graph->getProperty<LayoutProperty>("viewLayout");
  ColorProperty  *color  = graph->getProperty<ColorProperty>("viewColor");
  
  DataSet colorParam;
  colorParam.set("colormodel", 1);
  colorParam.set("color1", Color(255, 255, 0, 100));
  colorParam.set("color2", Color(0, 0, 255, 100));


  
  DataSet layoutParam;
  layoutParam.set("use length", true);
  layoutParam.set("initialisation", false);  
  layoutParam.set("initial layout", layout);
  layoutParam.set("3D layout", true);


  unsigned int img_number = 100000;
  for(unsigned int i = 2; i <= NB_MESURE; ++i) {
    GraphState *g_prev =  new GraphState(glW);
    stringstream sstr;
    sstr << "Mesure-" << i;
    cerr << "level : " << i << " ==> " << sstr.str() << endl;

    DoubleProperty *length = graph->getProperty<DoubleProperty>(sstr.str());
    colorParam.set("property", length);
    layoutParam.set("edge length", length);
    string msg;
    graph->computeProperty("Metric Mapping", color, msg, 0, &colorParam);
    cerr << "....metric mapping" << endl;
    //    color->setAllEdgeValue(Color(200,200,200,150));
    graph->computeProperty("GEM (Frick)", layout, msg, 0, &layoutParam);
    cerr << "... layout" << endl;
    glW->centerScene();

    Camera c = glW->getCamera();
    c.zoomFactor = 1;

    c.rotate(double(i-1) * M_PI/4.0, 0, 1, 0);
    glW->setCamera(c);

    GraphState * g_next =  new GraphState(glW);
    cerr << "1";
    Morphing morph;
    cerr << "2";
    morph.init(glW, g_prev, g_next);
    cerr << "3";
    float t = 0;
    cerr << "... morphing : " << endl;
    for (unsigned int k = 0; k<100; ++k) {
      morph.interpolate(glW, t);
      glW->draw();
      app->processEvents();
      app->processEvents();
      app->processEvents();
      QImage img = glW->grabFrameBuffer(false);
      stringstream sstr2;
      sstr2 << "img_" << img_number++ << ".jpg";
      img.save(sstr2.str(), "JPEG");
      t += 1.0 / 100.0;
      cerr << ".";
    }
    cerr << endl;
  }
}
/*******************************************************************/
int main(int argc,char ** argv ) {
  cerr << "ooooooooook" << endl;
  QApplication MainApp(argc,argv);
  
  if (argc<2) {
    cerr << "usage :" << endl;
    cerr << "\t " << argv[0] << " <filename>" << endl;
    cerr << "file must be in tlp format" << endl;
    exit(1);
  }

  /****************************************************/
  tlp::initTulipLib();
  tlp::loadPlugins();   // library side plugins
  GlGraph::loadPlugins(); //Glyoh 

  /****************************************************/
  GlGraphWidget MainWin;
  MainApp.setMainWidget(&MainWin);
  MainWin.resize(800,800);
  MainWin.show();
  
  if( argv[1][0] !='g' || argv[1][1] !='r' || argv[1][2] !='i' || argv[1][3] !='d'){
    importGraph(argv[1], &MainWin);
  }
  
  //  MainWin.setIncrementalRendering(false);
  MainWin.setEdgeColorInterpolate(false);
  MainWin.centerScene();
  MainWin.draw();
  MainApp.processEvents();
  MainApp.processEvents();

  makeFilm(&MainWin, &MainApp);

  return MainApp.exec();
}
