#include <algorithm>
#include <tulip/Graph.h>
#include <tulip/LayoutProperty.h>
#include <tulip/SizeProperty.h>
#include "TreeTools.h"

using namespace tlp;

//====================================================================
node searchRoot(const Graph* tree) {
    node            root;
    Iterator<node>* itNode = tree->getNodes();
    while (itNode->hasNext()) {
        node currentNode = itNode->next();
        if (tree->indeg(currentNode) == 0) {
            root = currentNode;
            break;
        }
    }
    delete itNode;
    return root;
}

//====================================================================
int getTreeDepth(const Graph* tree, node root) {
    int depthMax           = 0;
    Iterator<node>* itNode = tree->getOutNodes(root);
    while (itNode->hasNext())
        depthMax = std::max(getTreeDepth(tree, itNode->next()), depthMax);
	delete itNode;
    return depthMax + 1;
}

//====================================================================
bool isLeaf (const Graph* tree, node n) {
    return tree->outdeg(n) == 0;
}

//====================================================================
float getNodeX(LayoutProperty* pLayout, node current) {
    return pLayout->getNodeValue(current).getX();
}

//====================================================================
float getNodeY(LayoutProperty* pLayout, node current) {
    return pLayout->getNodeValue(current).getY();
}

//====================================================================
float getNodeHeight(SizeProperty* size, node current) {
    return size->getNodeValue(current).getH();
}

//====================================================================
float getNodeWidth(SizeProperty* size, node current) {
    return size->getNodeValue(current).getW();
}
