#include "PlanarConMapTest.h"
#include <tulip/PlanarityTest.h>
#include <cppunit/TestCase.h>
#include <cppunit/TestCaller.h>

#include <cppunit/extensions/HelperMacros.h>
CPPUNIT_TEST_SUITE_REGISTRATION( tlp::PlanarConMapTest );

using namespace std;
using namespace tlp;

//============================================================
// PlanarConMapTest
//============================================================
void PlanarConMapTest::build(){                                             
  graph->clear();edges.clear();nodes.clear();

  for(unsigned int i = 0; i < 4;++i)
    nodes.push_back(graph->addNode());

  edges.push_back(graph->addEdge(nodes[0], nodes[1]));
  edges.push_back(graph->addEdge(nodes[1], nodes[2]));
  edges.push_back(graph->addEdge(nodes[2], nodes[3]));
  edges.push_back(graph->addEdge(nodes[3], nodes[0])); 
  
  delete carte;
  carte = new PlanarConMap(graph);
}

//============================================================
void PlanarConMapTest::build2(){                                
  graph->clear();edges.clear();nodes.clear();

  for(unsigned int i = 0; i<6; ++i)
    nodes.push_back(graph->addNode());
    
  for(unsigned int i = 1; i <6; ++i)
    edges.push_back(graph->addEdge(nodes[0], nodes[i])); 

  delete carte;
  carte = new PlanarConMap(graph);
}

//============================================================
void PlanarConMapTest::build3(){                                             
  graph->clear();edges.clear();nodes.clear();
  for(unsigned int i = 0; i < 4 ; ++i)
    nodes.push_back(graph->addNode());
    
  edges.push_back(graph->addEdge(nodes[0], nodes[1])); 
  edges.push_back(graph->addEdge(nodes[1], nodes[2])); 
  edges.push_back(graph->addEdge(nodes[2], nodes[3])); 
  edges.push_back(graph->addEdge(nodes[3], nodes[0])); 
  edges.push_back(graph->addEdge(nodes[0], nodes[2])); 
  edges.push_back(graph->addEdge(nodes[1], nodes[3])); 

  delete carte;
  carte = new PlanarConMap(graph);
}

//============================================================
void PlanarConMapTest::build4() {
  graph->clear();edges.clear();nodes.clear();
  for(unsigned int i = 0; i < 10 ; ++i)
    nodes.push_back(graph->addNode());
  
  edges.push_back(graph->addEdge(nodes[0], nodes[1])); 
  edges.push_back(graph->addEdge(nodes[0], nodes[2])); 
  edges.push_back(graph->addEdge(nodes[0], nodes[3])); 
  edges.push_back(graph->addEdge(nodes[2], nodes[4])); 
  edges.push_back(graph->addEdge(nodes[2], nodes[5])); 
  edges.push_back(graph->addEdge(nodes[1], nodes[6])); 
  edges.push_back(graph->addEdge(nodes[6], nodes[7])); 
  edges.push_back(graph->addEdge(nodes[4], nodes[8])); 
  edges.push_back(graph->addEdge(nodes[4], nodes[9])); 

  delete carte;
  carte = new PlanarConMap(graph);
}

//============================================================
void PlanarConMapTest::testAddEdgeMap(){                                               

  /* test 1 */
  build();
  Face f1, f2;
  f1 = carte->faces[0];
  f2 = carte->faces[1];
  edge e = carte->addEdgeMap(nodes[0],nodes[2],f1,edges[0],edges[2]);
  Face f3 = carte->faces[2];
  vector<vector<edge> > cycles(5);
  cycles[0].push_back(edges[1]);      cycles[1].push_back(edges[2]);
  cycles[0].push_back(edges[0]);      cycles[1].push_back(edges[3]);
  cycles[0].push_back(e);             cycles[1].push_back(edges[0]);
                                      cycles[1].push_back(edges[1]);
  cycles[2].push_back(edges[3]);    
  cycles[2].push_back(edges[2]);    
  cycles[2].push_back(e);    

  for(unsigned int i = 0; i< 3; i++){
    vector<edge> tmp;
    Iterator<edge>* it;
    if(i==0)
      it = carte->getFaceEdges(f1);
    if(i==1)
      it = carte->getFaceEdges(f2);
    if(i==2)
      it = carte->getFaceEdges(f3);
    while(it->hasNext()){
      edge e_tmp = it->next();
      tmp.push_back(e_tmp);
    } delete it;
    CPPUNIT_ASSERT_MESSAGE(" test 1 AddEdgeMap cycle ",tmp == cycles[i]);
  }
  CPPUNIT_ASSERT_MESSAGE(" test 1 AddEdgeMap dispositif decorateur ",carte->numberOfEdges() == graph->numberOfEdges());

  /* test 2 */
  build2();
  f1 = carte->faces[0];
  e = carte->addEdgeMap(nodes[1],nodes[3],f1,edges[0],edges[2]);
  f2 = carte->faces[1];
  vector<vector<edge> > cycles2(2);
  cycles2[0].push_back(edges[2]);
  cycles2[0].push_back(edges[3]);
  cycles2[0].push_back(edges[3]);
  cycles2[0].push_back(edges[4]);
  cycles2[0].push_back(edges[4]);
  cycles2[0].push_back(edges[0]);
  cycles2[0].push_back(e);       
                                
  cycles2[1].push_back(edges[0]);    
  cycles2[1].push_back(edges[1]);    
  cycles2[1].push_back(edges[1]);    
  cycles2[1].push_back(edges[2]);    
  cycles2[1].push_back(e);    

  for(unsigned int i = 0; i< 2; i++){
    vector<edge> tmp;
    Iterator<edge>* it;
    if(i==0)
      it = carte->getFaceEdges(f1);
    if(i==1)
      it = carte->getFaceEdges(f2);
    while(it->hasNext()){
      edge e_tmp = it->next();
      tmp.push_back(e_tmp);
    } delete it;
    CPPUNIT_ASSERT_MESSAGE(" test 2 AddEdgeMap cycle ",tmp == cycles2[i]);
  }
  CPPUNIT_ASSERT_MESSAGE(" test 2 AddEdgeMap dispositif decorateur ",carte->numberOfEdges() == graph->numberOfEdges());

  /* test 3 */
  build4();
  f1 = carte->faces[0];
  e = carte->addEdgeMap(nodes[2],nodes[7],f1,edges[4],edges[6]);
  f2 = carte->faces[1];
  edges.push_back(e);
  e = carte->addEdgeMap(nodes[1],nodes[5],f1,edges[5],edges[4]);
  f3 = carte->faces[2];
  edges.push_back(e);
  e = carte->addEdgeMap(nodes[1],nodes[4],f2,edges[0],edges[3]);
  Face f4 = carte->faces[3];
  edges.push_back(e);
  vector<vector<edge> > cycles3(4);
  cycles3[0].push_back(edges[4]);
  cycles3[0].push_back(edges[9]);
  cycles3[0].push_back(edges[6]);
  cycles3[0].push_back(edges[5]);
  cycles3[0].push_back(edges[10]);

  cycles3[1].push_back(edges[8]);
  cycles3[1].push_back(edges[8]);
  cycles3[1].push_back(edges[7]);
  cycles3[1].push_back(edges[7]);
  cycles3[1].push_back(edges[3]);
  cycles3[1].push_back(edges[1]);
  cycles3[1].push_back(edges[2]);
  cycles3[1].push_back(edges[2]);
  cycles3[1].push_back(edges[0]);
  cycles3[1].push_back(edges[11]);

  cycles3[2].push_back(edges[0]);
  cycles3[2].push_back(edges[1]);
  cycles3[2].push_back(edges[4]);
  cycles3[2].push_back(edges[10]);

  cycles3[3].push_back(edges[5]);
  cycles3[3].push_back(edges[6]);
  cycles3[3].push_back(edges[9]);
  cycles3[3].push_back(edges[3]);
  cycles3[3].push_back(edges[11]);
  for(unsigned int i = 0; i< 4; ++i){
    vector<edge> tmp;
    Iterator<edge>* it;
    if(i==0)
      it = carte->getFaceEdges(f1);
    if(i==1)
      it = carte->getFaceEdges(f2);
    if(i==2)
      it = carte->getFaceEdges(f3);
    if(i==3)
      it = carte->getFaceEdges(f4);
    while(it->hasNext()){
      edge e_tmp = it->next();
      tmp.push_back(e_tmp);
    } delete it;
    CPPUNIT_ASSERT_MESSAGE(" test 2 AddEdgeMap cycle ",tmp == cycles3[i]);
  }
  
  CPPUNIT_ASSERT_MESSAGE(" test 2 AddEdgeMap dispositif decorateur ",carte->numberOfEdges() == graph->numberOfEdges());
}

//============================================================
void PlanarConMapTest::testDelEdgeMap(){
  build();
  Face f1, f2;
  f1 = carte->faces[0];
  f2 = carte->faces[1];
  edge e = carte->addEdgeMap(nodes[0],nodes[2],f1,edges[0],edges[2]);

  carte->delEdgeMap(e);
  f1 = carte->faces[0];
  f2 = carte->faces[1];

  vector<vector<edge> > cycles(2);
  cycles[1].push_back(edges[3]);      cycles[0].push_back(edges[2]);
  cycles[1].push_back(edges[2]);      cycles[0].push_back(edges[3]);
  cycles[1].push_back(edges[1]);      cycles[0].push_back(edges[0]);
  cycles[1].push_back(edges[0]);      cycles[0].push_back(edges[1]);

  for(unsigned int i = 0; i< 2; i++){
    vector<edge> tmp;
    Iterator<edge>* it;
    if(i==0)
      it = carte->getFaceEdges(f1);
    if(i==1)
      it = carte->getFaceEdges(f2);
    while(it->hasNext()){
      edge e_tmp = it->next();
      tmp.push_back(e_tmp);
    } delete it;
    CPPUNIT_ASSERT_MESSAGE(" test DelEdgeMap cycle ",tmp == cycles[i]);
  }
  
  CPPUNIT_ASSERT_MESSAGE(" test AddEdgeMap dispositif decorateur ",carte->numberOfEdges() == graph->numberOfEdges());
  carte->clear();  
}

//============================================================
void PlanarConMapTest::testNbFaces(){                                                    

  build();
  CPPUNIT_ASSERT_MESSAGE(" test nbFaces ",2 == carte->nbFaces());
  carte->clear();
}

//============================================================
void PlanarConMapTest::testUpdate(){                                                  

  build();
  
  unsigned int tmp = carte->nbFaces();
  edge e = graph->addEdge(nodes[0],nodes[2]);
  carte->update();
  CPPUNIT_ASSERT_MESSAGE(" test update() ", (tmp+1) == carte->nbFaces());   
}

//============================================================
void PlanarConMapTest::testMergeFaces(){
  build();
  Face f1, f2;
  f1 = carte->faces[0];
  f2 = carte->faces[1];
  edge e = carte->addEdgeMap(nodes[0],nodes[2],f1,edges[0],edges[2]);
  
  carte->mergeFaces(carte->edgesFaces[e][0],carte->edgesFaces[e][1]);
  f1 = carte->faces[0];
  f2 = carte->faces[1];
  
  vector<vector<edge> > cycles(2);
  cycles[1].push_back(edges[3]);      cycles[0].push_back(edges[2]);
  cycles[1].push_back(edges[2]);      cycles[0].push_back(edges[3]);
  cycles[1].push_back(edges[1]);      cycles[0].push_back(edges[0]);
  cycles[1].push_back(edges[0]);      cycles[0].push_back(edges[1]);
  
  for(unsigned int i = 0; i< 2; i++){
    vector<edge> tmp;
    Iterator<edge>* it;
    if(i==0)
      it = carte->getFaceEdges(f1);
    if(i==1)
      it = carte->getFaceEdges(f2);
    while(it->hasNext()){
      edge e_tmp = it->next();
      tmp.push_back(e_tmp);
    } delete it;
    CPPUNIT_ASSERT_MESSAGE(" test DelEdgeMap cycle ",tmp == cycles[i]);
  }
  
  CPPUNIT_ASSERT_MESSAGE(" test AddEdgeMap dispositif decorateur ",carte->numberOfEdges() == graph->numberOfEdges());
  carte->clear();  

}
//============================================================
void PlanarConMapTest::testSplitFace(){

  /* test 1 */
  build();
  Face f1, f2;
  f1 = carte->faces[0];
  f2 = carte->faces[1];
  Face f3 = carte->splitFace(f1,nodes[0],nodes[2]);
  edge e = carte->existEdge(nodes[0],nodes[2]).isValid() ? carte->existEdge(nodes[0],nodes[2]) : carte->existEdge(nodes[2],nodes[0]);
  vector<vector<edge> > cycles(5);
  cycles[0].push_back(edges[1]);      cycles[1].push_back(edges[2]);
  cycles[0].push_back(edges[0]);      cycles[1].push_back(edges[3]);
  cycles[0].push_back(e);             cycles[1].push_back(edges[0]);
                                      cycles[1].push_back(edges[1]);
  cycles[2].push_back(edges[3]);    
  cycles[2].push_back(edges[2]);    
  cycles[2].push_back(e);    

  for(unsigned int i = 0; i< 3; i++){
    vector<edge> tmp;
    Iterator<edge>* it;
    if(i==0)
      it = carte->getFaceEdges(f1);
    if(i==1)
      it = carte->getFaceEdges(f2);
    if(i==2)
      it = carte->getFaceEdges(f3);
    while(it->hasNext()){
      edge e_tmp = it->next();
      tmp.push_back(e_tmp);
    } delete it;
    CPPUNIT_ASSERT_MESSAGE(" test AddEdgeMap cycle ",tmp == cycles[i]);
  }
  CPPUNIT_ASSERT_MESSAGE(" test AddEdgeMap dispositif decorateur ",carte->numberOfEdges() == graph->numberOfEdges());
    
  /* test 2 */
  build4();
  //  cout << carte<< endl;
  f1 = carte->faces[0];
  f2 = carte->splitFace(f1,nodes[2],nodes[7]);
  e = carte->existEdge(nodes[2],nodes[7]).isValid() ? carte->existEdge(nodes[2],nodes[7]) : carte->existEdge(nodes[7],nodes[2]);
  edges.push_back(e);
  //  cout << carte<< endl;
  f3 = carte->splitFace(f2,nodes[1],nodes[5]);
  e = carte->existEdge(nodes[1],nodes[5]).isValid() ? carte->existEdge(nodes[1],nodes[5]) : carte->existEdge(nodes[5],nodes[1]);
  edges.push_back(e);
  //  cout << carte<< endl;
  Face f4 = carte->splitFace(f3,nodes[1],nodes[4]);
  e = carte->existEdge(nodes[1],nodes[4]).isValid() ? carte->existEdge(nodes[1],nodes[4]) : carte->existEdge(nodes[4],nodes[1]);
  edges.push_back(e);
  //  cout << carte<< endl;
  vector<vector<edge> > cycles3(4);
  cycles3[0].push_back(edges[4]);
  cycles3[0].push_back(edges[9]);
  cycles3[0].push_back(edges[6]);
  cycles3[0].push_back(edges[5]);
  cycles3[0].push_back(edges[10]);

  cycles3[1].push_back(edges[8]);
  cycles3[1].push_back(edges[8]);
  cycles3[1].push_back(edges[7]);
  cycles3[1].push_back(edges[7]);
  cycles3[1].push_back(edges[3]);
  cycles3[1].push_back(edges[1]);
  cycles3[1].push_back(edges[2]);
  cycles3[1].push_back(edges[2]);
  cycles3[1].push_back(edges[0]);
  cycles3[1].push_back(edges[11]);

  cycles3[2].push_back(edges[0]);
  cycles3[2].push_back(edges[1]);
  cycles3[2].push_back(edges[4]);
  cycles3[2].push_back(edges[10]);

  cycles3[3].push_back(edges[5]);
  cycles3[3].push_back(edges[6]);
  cycles3[3].push_back(edges[9]);
  cycles3[3].push_back(edges[3]);
  cycles3[3].push_back(edges[11]);
  for(unsigned int i = 0; i< 4; ++i){
    vector<edge> tmp;
    Iterator<edge>* it;
    if(i==0)
      it = carte->getFaceEdges(f1);
    if(i==1)
      it = carte->getFaceEdges(f2);
    if(i==2)
      it = carte->getFaceEdges(f3);
    if(i==3)
      it = carte->getFaceEdges(f4);
    while(it->hasNext()){
      edge e_tmp = it->next();
      tmp.push_back(e_tmp);
    } delete it;
    CPPUNIT_ASSERT_MESSAGE(" test 2 AddEdgeMap cycle ",tmp == cycles3[i]);
  }
  
  CPPUNIT_ASSERT_MESSAGE(" test 2 AddEdgeMap dispositif decorateur ",carte->numberOfEdges() == graph->numberOfEdges());

}

//============================================================
void PlanarConMapTest::testSuccCycleEdge(){                                       

  build2();

  vector<edge> cycles;
  cycles.push_back(edges[0]);
  cycles.push_back(edges[1]);
  cycles.push_back(edges[2]);
  cycles.push_back(edges[3]);
  cycles.push_back(edges[4]); 

  
  unsigned int i = 0;
  Iterator<edge>* it = carte->getInOutEdges(nodes[0]);
  edge e = it->next();
  delete it;  
  CPPUNIT_ASSERT_MESSAGE(" test SuccCycleEdge deb",e == cycles[0]);
  do{
    e = carte->succCycleEdge(e,nodes[0]);
    i++;
    CPPUNIT_ASSERT_MESSAGE(string(" test SuccCycleEdge cycle "),e == cycles[i]);
  }
  while(e != edges[0] && i<4);
  carte->clear();
}
   
//============================================================
void PlanarConMapTest::testPrecCycleEdge(){                                     

  build2();

  vector<edge> cycles;
  cycles.push_back(edges[0]);
  cycles.push_back(edges[4]);
  cycles.push_back(edges[3]);
  cycles.push_back(edges[2]);
  cycles.push_back(edges[1]);
  
  unsigned int i = 0;
  Iterator<edge>* it = carte->getInOutEdges(nodes[0]);
  edge e = it->next();
  delete it;
  CPPUNIT_ASSERT_MESSAGE(" test predCycleEdge deb",e == cycles[0]);
  do{
    e = carte->predCycleEdge(e,nodes[0]);
    i++;
    CPPUNIT_ASSERT_MESSAGE(string(" test predCycleEdge cycle "),e == cycles[i]);
  }
  while((e != edges[0]) && (i<4));
  carte->clear();
}  

//============================================================                         
void PlanarConMapTest::testComputeFaces(){

  build();
  int fc = carte->nbFaces();
  int e = carte->numberOfEdges();
  int n = carte->numberOfNodes();
  CPPUNIT_ASSERT_MESSAGE(" test ComputesFaces 1", fc == e - n + 2);
  carte->clear();

  build2();
  fc = carte->nbFaces();
  e = carte->numberOfEdges();
  n = carte->numberOfNodes();
  CPPUNIT_ASSERT_MESSAGE(" test ComputesFaces 2", fc == e - n + 2);
  carte->clear();

  build3();
  fc = carte->nbFaces();
  e = carte->numberOfEdges();
  n = carte->numberOfNodes();
  CPPUNIT_ASSERT_MESSAGE(" test ComputesFaces 3 ", fc == e - n + 2);
  carte->clear();
}

//============================================================
CppUnit::Test* PlanarConMapTest::suite(){                    
  CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite;
  suiteOfTests->addTest(new CppUnit::TestCaller<PlanarConMapTest>("testComputeFaces", &PlanarConMapTest::testComputeFaces));
  suiteOfTests->addTest(new CppUnit::TestCaller<PlanarConMapTest>("testNbFaces", &PlanarConMapTest::testNbFaces));
  suiteOfTests->addTest(new CppUnit::TestCaller<PlanarConMapTest>("testSuccCycleEdge", &PlanarConMapTest::testSuccCycleEdge));
  suiteOfTests->addTest(new CppUnit::TestCaller<PlanarConMapTest>("testPrecCycleEdge", &PlanarConMapTest::testPrecCycleEdge));
  suiteOfTests->addTest(new CppUnit::TestCaller<PlanarConMapTest>("testAddEdgeMap", &PlanarConMapTest::testAddEdgeMap));
  suiteOfTests->addTest(new CppUnit::TestCaller<PlanarConMapTest>("testAddEdgeMap", &PlanarConMapTest::testDelEdgeMap));
  suiteOfTests->addTest(new CppUnit::TestCaller<PlanarConMapTest>("testSplitFace", &PlanarConMapTest::testMergeFaces));
  suiteOfTests->addTest(new CppUnit::TestCaller<PlanarConMapTest>("testSplitFace", &PlanarConMapTest::testSplitFace));
  suiteOfTests->addTest(new CppUnit::TestCaller<PlanarConMapTest>("testUpdate", &PlanarConMapTest::testUpdate));
  return suiteOfTests;
}
//============================================================
